"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.onOpenStreamChannel = exports.openStream = exports.setNamespace = exports.allowWindowMessaging = exports.onMessage = exports.sendMessage = exports.isInternalEnpoint = void 0;
const events_1 = require("events");
const tiny_uid_1 = __importDefault(require("tiny-uid"));
const Stream_1 = require("./Stream");
const internal_1 = require("./internal");
Object.defineProperty(exports, "sendMessage", { enumerable: true, get: function () { return internal_1.sendMessage; } });
Object.defineProperty(exports, "onMessage", { enumerable: true, get: function () { return internal_1.onMessage; } });
Object.defineProperty(exports, "allowWindowMessaging", { enumerable: true, get: function () { return internal_1.allowWindowMessaging; } });
Object.defineProperty(exports, "setNamespace", { enumerable: true, get: function () { return internal_1.setNamespace; } });
Object.defineProperty(exports, "isInternalEnpoint", { enumerable: true, get: function () { return internal_1.isInternalEnpoint; } });
const openStreams = new Map();
const onOpenStreamCallbacks = new Map();
const streamyEmitter = new events_1.EventEmitter();
internal_1.onMessage('__crx_bridge_stream_open__', (message) => {
    return new Promise((resolve) => {
        const { sender, data } = message;
        const { channel } = data;
        let watching = false;
        const readyup = () => {
            const callback = onOpenStreamCallbacks.get(channel);
            if (typeof callback === 'function') {
                callback(new Stream_1.Stream({ ...data, endpoint: sender }));
                if (watching) {
                    streamyEmitter.removeListener('did-change-stream-callbacks', readyup);
                }
                resolve(true);
            }
            else if (!watching) {
                watching = true;
                streamyEmitter.on('did-change-stream-callbacks', readyup);
            }
        };
        readyup();
    });
});
async function openStream(channel, destination) {
    if (openStreams.has(channel)) {
        throw new Error('crx-bridge: A Stream is already open at this channel');
    }
    const endpoint = typeof destination === 'string' ? internal_1.parseEndpoint(destination) : destination;
    const streamInfo = { streamId: tiny_uid_1.default(), channel, endpoint };
    const stream = new Stream_1.Stream(streamInfo);
    stream.onClose(() => openStreams.delete(channel));
    await internal_1.sendMessage('__crx_bridge_stream_open__', streamInfo, endpoint);
    openStreams.set(channel, stream);
    return stream;
}
exports.openStream = openStream;
function onOpenStreamChannel(channel, callback) {
    if (onOpenStreamCallbacks.has(channel)) {
        throw new Error('crx-bridge: This channel has already been claimed. Stream allows only one-on-one communication');
    }
    onOpenStreamCallbacks.set(channel, callback);
    streamyEmitter.emit('did-change-stream-callbacks');
}
exports.onOpenStreamChannel = onOpenStreamChannel;
//# sourceMappingURL=Bridge.js.map