import { JsonValue } from 'type-fest';
declare enum RuntimeContext {
    Devtools = "devtools",
    Background = "background",
    ContentScript = "content-script",
    Window = "window"
}
export declare type Endpoint = {
    context: RuntimeContext;
    tabId: number;
};
export interface IBridgeMessage<T extends JsonValue> {
    sender: Endpoint;
    id: string;
    data: T;
    timestamp: number;
}
export declare type OnMessageCallback<T extends JsonValue> = (message: IBridgeMessage<T>) => void | JsonValue | Promise<JsonValue>;
export declare const parseEndpoint: (endpoint: string) => Endpoint;
export declare const isInternalEnpoint: ({ context: ctx }: Endpoint) => boolean;
/**
 * Sends a message to some other endpoint, to which only one listener can send response.
 * Returns Promise. Use `then` or `await` to wait for the response.
 * If destination is `window` message will routed using window.postMessage.
 * Which requires a shared namespace to be set between `content-script` and `window`
 * that way they can recognize each other when global window.postMessage happens and there are other
 * extensions using crx-bridge as well
 * @param messageID
 * @param data
 * @param destination
 */
export declare function sendMessage<T extends JsonValue>(messageID: string, data: JsonValue, destination: string | Endpoint): Promise<T>;
export declare function onMessage<T extends JsonValue>(messageID: string, callback: OnMessageCallback<T>): void;
export declare function setNamespace(nsps: string): void;
export declare function allowWindowMessaging(nsps: string): void;
export {};
